module Vrmul8x8p(X, Y, P);
  input [7:0] X, Y;
  output [15:0] P;
  reg [7:0] PC [0:7];         // product-component bits
  reg [7:0] PCS [0:7];        // full-adder sum bits
  reg [7:0] PCC [0:7];        // full-adder carry output bits
  reg [6:0] RAS;              // ripple adder sum 
  reg [7:0] RAC;              //   and carry bits
  reg [7:0] T, PCS7, PCC7;    // temporary variables
  reg [15:0] P;               // output variable for assignment
  integer i;

  function MAJ;
    input I1, I2, I3;
      MAJ = (I1 & I2) | (I1 & I3) | (I2 & I3);
  endfunction 

  function [7:0] MAJ8;
    input [7:0] I1, I2, I3;
      MAJ8 = (I1 & I2) | (I1 & I3) | (I2 & I3);
  endfunction  

  always @ (X or Y) begin
    for (i=0; i<=7; i=i+1)
      PC[i] = Y[i] ? X : 8'b0;   // Compute product-component bits
    PCS[0] = PC[0];         // Set up outputs of first-row "virtual"
    PCC[0] = 8'b0;          //   full adders (not shown in figure).
    for (i=1; i<=7; i=i+1) begin  // Do all "real" full adders.
      PCS[i] = PC[i] ^ (PCS[i-1]>>1) ^ PCC[i-1];  
      PCC[i] = MAJ8(PC[i], PCS[i-1]>>1, PCC[i-1]);
    end
    PCS7 = PCS[7];    // Temp needed to access individual bits
    PCC7 = PCC[7];    //    of PCS[7]; ditto for PCC[7].
    RAC[0] = 1'b0;    // No carry into final ripple adder.
    for (i=0; i<=6; i=i+1) begin               // Final ripple-adder 
      RAS[i] = PCS7[i+1] ^ PCC7[i] ^ RAC[i];   //     sum 
      RAC[i+1] = MAJ(PCS7[i+1], PCC7[i], RAC[i]); //  and carry bits
    end
    for (i=0; i<=7; i=i+1) begin
      T = PCS[i];
      P[i] = T[0];  // first 8 product bits from full-adder sums
    end
    for (i=8; i<=14; i=i+1)
      P[i] = RAS[i-8];   // next 7 bits from ripple-adder sums
    P[15] = RAC[7];      // last bit from ripple-adder carry out
  end
endmodule
