module Vrmul8x8s(X, Y, P);
  input [7:0] X, Y;
  output [15:0] P;
  wire [7:0] PC [0:7];         // product-component bits
  wire [7:0] PCS [0:7];        // full-adder sum bits
  wire [7:0] PCC [0:7];        // full-adder carry output bits
  wire [6:0] RAS;              // ripple adder sum 
  wire [7:0] RAC;              //   and carry bits
  genvar i;

  assign PCS[0] = PC[0];   // Set up outputs of first-row "virtual"
  assign PCC[0] = 8'b0;    //   full adders (not shown in figure).
  assign RAC[0] = 1'b0;    // No carry into final ripple adder.

  generate
    for (i=0; i<=7; i=i+1) begin: PCgen
      assign PC[i] = {8{Y[i]}} & X;  // Compute product-component bits
    end

    for (i=1; i<=7; i=i+1) begin: FAgen   // Do all "real" full adders.  
      assign PCS[i] = PC[i] ^ PCS[i-1]>>1 ^ PCC[i-1];  
      Maj #(8) M_FA ( PCC[i], PC[i], PCS[i-1]>>1, PCC[i-1] );
    end

    for (i=0; i<=6; i=i+1) begin: RAgen   // Final ripple-adder 
      xor X_RA ( RAS[i], PCS[7][i+1], PCC[7][i], RAC[i] );   //  sum 
      Maj M_RA ( RAC[i+1], PCS[7][i+1], PCC[7][i], RAC[i] ); //  carry 
    end

    for (i=0; i<=7; i=i+1) begin: Pgen0_7
      assign P[i] = PCS[i][0]; // first 8 prod bits from full-adder sums
    end

    for (i=8; i<=14; i=i+1) begin: Pgen8_14
      assign P[i] = RAS[i-8]; // next 7 bits from ripple-adder sums
    end
  endgenerate

  assign P[15] = RAC[7];      // last bit from ripple-adder carry out

endmodule			
